/* Variables */
  @import 'variables.css';

/* Default settings */
  
  /* all */
  * {
    margin:0;
    padding:0;
    box-sizing: border-box;
    -webkit-text-size-adjust:100%;
  }
  a, a:visited {
    color:#0000EE;
  }
  
  /* header */
  .header {
    margin:0 auto;
    background:#fff;
    margin-bottom:15px;
    border-bottom:1px solid #D9D9D9;
  }
  
  /* footer */
  .footer {
    background:#1A1A1A;
    margin-top:15px;
    text-align:center;
    padding:15px 0;
    border-top:1px solid #ccc;
  }
  .footer a, .footer a:visited {
    color:white;
  }
  
  /* container */
  .container {
    background:#F2EFE;
  }
  
  /* main */
  .main {
    max-width:1030px;
    width:100%;
    margin: 0 auto;
    padding:0 5px;
    display:flex;
    flex-wrap:wrap;
  }

  /* content */
  .content {
    width:calc(100% - 320px - 30px);
  }
  
  /* entry */
  .entry {
    background:white;
    margin-bottom:10px;
  }
  .entry li {
    margin-left:25px;
    line-height:1.5;
  }
  
  /* side */
  .side {
    width:320px;
    margin-left:30px;
    padding-top:16px;
  }
  
  /* tags */
  h1 {
    max-width:1030px;
    width:100%;
    font-size:28px;
    padding:19px 10px 15px 73px;
    background:url(/images/logo.png) no-repeat 10px 15px;
    background-size:48px;
  }
  h1 a, h1 a:visited {
    color:white;
    text-decoration:none;
    color:#333;
  }
  h2 {
    font-size:24px;
    margin:10px 0;
    padding-left:20px;
    letter-spacing:0.5px;
    border-left:8px solid var(--theme_color);
  }
  h2 a, h2 a:visited {
    text-decoration:none;
    color:#000;
  }
  h3 {
    font-size:20px;
    border-top:4px solid var(--theme_color);
    padding:16px 0px 15px 18px;
    letter-spacing:0.5px;
    margin:15px 0 10px 0;
    background:#F5F5F5;
    margin-top:30px;
  }
  h4 {
    font-size:20px;
    font-weight:bold;
    padding:6px 0px 4px 5px;
    letter-spacing:0.5px;
    margin:15px 0 18px 0;
    border-bottom:dashed 1px #ddd;
  }
  p {
    font-size:16px;
    color:#111;
    margin:10px 0;
    line-height:1.5;
  }
  pre {
    font-size:16px;
    background:#FCFCFC;
    border:1px solid #CCCCCC;
    border-radius:5px;
    padding:10px 15px;
    margin:10px 0;
    line-height:1.5;
    white-space:pre-wrap;
  }
  blockquote {
    font-size:16px;
    background:#FCFCFC;
    border:1px solid #CCCCCC;
    border-radius:5px;
    padding:10px 15px;
    margin:10px 0;
    line-height:1.5;
  }

  table {
    border-collapse: collapse;
    margin-left:15px;
  }

  table th {
    border:1px solid #ddd;
    padding:7px;
    background:#eee;
    font-weight:bold;
  }

  table td {
    border:1px solid #ddd;
    padding:7px;
  }
  
  /* advanced */
  .day {
    text-align:right;
    color:#999;
  }
  .before-days {
    text-align:center;
  }
  .side_list {
    margin-bottom:30px;
  }
  .side_list_title {
    background:var(--theme_color);
    color:white;
    font-weight:bold;
    padding: 12px 0px 10px 0px;
    text-align:center;
    border:1px solid #009D82;
  }
  .side_list ul {
    border:1px solid #ccc;
  }
  .side_list li {
    list-style:none;
    background:white;
    border-bottom:1px solid #E5E5E5;
    padding:12px 0px 10px 15px;
  }
  .side li:last-child {
    border-bottom:none;
  }

/* Tablet and Smart phone */
@media only screen and (max-width:1030px) {
  .content {
    width:97%;
    margin:0 auto;
    margin-bottom:15px;
  }
  .side {
    width:97%;
    margin:0 auto;
  }
}
