<!-- side -->
<div class="side_list">
  <div class="side_list_title">
    Side Bar
  </div>
  <ul>
    <li><a href="/list.html">Entries</a></li>
    <li>Bar1</li>
    <li>Baz1</li>
  </ul>
</div>

<div class="side_list">
  <div class="side_list_title">
    Side Bar2
  </div>
  <ul>
    <li>Bar2</li>
    <li>Baz2</li>
  </ul>
</div>
